@@ -18,6 +18,9 @@
 # Free Software Foundation, Inc., Franklin Street, Fifth Floor,
 # Boston MA  02110-1301 USA.
 
+# pylint: disable=broad-exception-caught
+# pylint: disable=wrong-import-position
+
 """
 Utilities for obtaining information about accessible objects.
 These utilities are app-type- and toolkit-agnostic. Utilities that might have
@@ -51,6 +54,7 @@ class AXObject:
     KNOWN_DEAD = {}
     REAL_APP_FOR_MUTTER_FRAME = {}
     REAL_FRAME_FOR_MUTTER_FRAME = {}
+    OBJECT_ATTRIBUTES = {}
 
     _lock = threading.Lock()
 
@@ -60,22 +64,41 @@ class AXObject:
 
         while True:
             time.sleep(60)
-            with AXObject._lock:
-                tokens = ["AXObject: Clearing known dead-or-alive state for",
-                          len(AXObject.KNOWN_DEAD), "objects"]
-                debug.printTokens(debug.LEVEL_INFO, tokens, True)
-                AXObject.KNOWN_DEAD.clear()
+            AXObject._clear_all_dictionaries()
 
-                tokens = ["AXObject: Clearing", len(AXObject.REAL_APP_FOR_MUTTER_FRAME),
-                          "real apps for mutter frames"]
-                debug.printTokens(debug.LEVEL_INFO, tokens, True)
-                AXObject.REAL_APP_FOR_MUTTER_FRAME.clear()
+    @staticmethod
+    def _clear_all_dictionaries(reason=""):
+        msg = "AXObject: Clearing local cache."
+        if reason:
+            msg += f" Reason: {reason}"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
-                tokens = ["AXObject: Clearing", len(AXObject.REAL_FRAME_FOR_MUTTER_FRAME),
-                          "real frames for mutter frames"]
-                debug.printTokens(debug.LEVEL_INFO, tokens, True)
-                AXObject.REAL_FRAME_FOR_MUTTER_FRAME.clear()
+        with AXObject._lock:
+            tokens = ["AXObject: Clearing known dead-or-alive state for",
+                        len(AXObject.KNOWN_DEAD), "objects"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            AXObject.KNOWN_DEAD.clear()
 
+            tokens = ["AXObject: Clearing", len(AXObject.REAL_APP_FOR_MUTTER_FRAME),
+                        "real apps for mutter frames"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            AXObject.REAL_APP_FOR_MUTTER_FRAME.clear()
+
+            tokens = ["AXObject: Clearing", len(AXObject.REAL_FRAME_FOR_MUTTER_FRAME),
+                        "real frames for mutter frames"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            AXObject.REAL_FRAME_FOR_MUTTER_FRAME.clear()
+
+            tokens = ["AXObject: Clearing cached object attributes for",
+                        len(AXObject.OBJECT_ATTRIBUTES), "objects"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            AXObject.OBJECT_ATTRIBUTES.clear()
+
+    @staticmethod
+    def clear_cache_now(reason=""):
+        """Clears all cached information immediately."""
+
+        AXObject._clear_all_dictionaries(reason)
 
     @staticmethod
     def start_cache_clearing_thread():
@@ -85,6 +108,23 @@ class AXObject:
         thread.daemon = True
         thread.start()
 
+    @staticmethod
+    def is_bogus(obj):
+        """Hack to ignore certain objects. All entries must have a bug."""
+
+        # TODO - JD: Periodically check for fixes and remove hacks which are no
+        # longer needed.
+
+        # https://bugzilla.mozilla.org/show_bug.cgi?id=1879750
+        if AXObject.get_role(obj) == Atspi.Role.SECTION \
+           and AXObject.get_role(AXObject.get_parent(obj)) == Atspi.Role.FRAME \
+           and Atspi.Accessible.get_toolkit_name(obj).lower() == "gecko":
+            tokens = ["AXObject:", obj, "is bogus. See mozilla bug 1879750."]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True, True)
+            return True
+
+        return False
+
     @staticmethod
     def is_valid(obj):
         """Returns False if we know for certain this object is invalid"""
@@ -116,7 +156,7 @@ class AXObject:
 
         if current_status:
             tokens = ["AXObject: Removing", obj, "from known-dead objects"]
-            debug.printTokens(debug.LEVEL_INFO, msg, tokens, True)
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
     @staticmethod
     def handle_error(obj, error, msg):
@@ -432,6 +472,11 @@ class AXObject:
             debug.printTokens(debug.LEVEL_INFO, tokens, True)
             return None
 
+        if parent is None \
+           and AXObject.get_role(obj) not in [Atspi.Role.INVALID, Atspi.Role.DESKTOP_FRAME]:
+            tokens = ["AXObject:", obj, "claims to have no parent"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+
         return parent
 
     @staticmethod
@@ -584,11 +629,12 @@ class AXObject:
 
         for i in range(AXObject.get_child_count(obj)):
             child = AXObject.get_child_checked(obj, i)
-            if child and pred(child):
+            if child is None:
+                continue
+            if pred(child):
                 return child
-
             child = AXObject._find_descendant(child, pred)
-            if child and pred(child):
+            if child:
                 return child
 
         return None
@@ -660,18 +706,29 @@ class AXObject:
             AXObject.handle_error(obj, error, msg)
             return Atspi.Role.INVALID
 
+        # Handle the fact that GTK4 is not following the Core-AAM mappings and is instead exposing
+        # roles that should be exposed as PANEL as GROUPING. This is not a problem in Orca v48
+        # because it has explicit handling for the GROUPING role. Back porting the necessary logic
+        # to v47 is a bigger change than I would like to make for a stable version. Therefore if
+        # we are given a GROUPING, pretend we were given a PANEL.
+        if role == Atspi.Role.GROUPING:
+            role = Atspi.Role.PANEL
+
         AXObject._set_known_dead_status(obj, False)
         return role
 
     @staticmethod
-    def get_role_name(obj):
+    def get_role_name(obj, localized=False):
         """Returns the accessible role name of obj"""
 
         if not AXObject.is_valid(obj):
             return ""
 
         try:
-            role_name = Atspi.Accessible.get_role_name(obj)
+            if not localized:
+                role_name = Atspi.Accessible.get_role_name(obj)
+            else:
+                role_name = Atspi.Accessible.get_localized_role_name(obj)
         except Exception as error:
             msg = f"AXObject: Exception in get_role_name: {error}"
             AXObject.handle_error(obj, error, msg)
@@ -722,6 +779,55 @@ class AXObject:
 
         return description
 
+    @staticmethod
+    def get_image_description(obj):
+        """Returns the accessible image description of obj"""
+
+        if not AXObject.supports_image(obj):
+            return ""
+
+        try:
+            description = Atspi.Image.get_image_description(obj)
+        except Exception as error:
+            msg = f"AXObject: Exception in get_image_description: {error}"
+            AXObject.handle_error(obj, error, msg)
+            return ""
+
+        return description
+
+    @staticmethod
+    def get_image_size(obj):
+        """Returns a (width, height) tuple of the image in obj"""
+
+        if not AXObject.supports_image(obj):
+            return 0, 0
+
+        try:
+            result = Atspi.Image.get_image_size(obj)
+        except Exception as error:
+            msg = f"AXObject: Exception in get_image_size: {error}"
+            AXObject.handle_error(obj, error, msg)
+            return 0, 0
+
+        # The return value is an AtspiPoint, hence x and y.
+        return result.x, result.y
+
+    @staticmethod
+    def get_help_text(obj):
+        """Returns the accessible help text of obj"""
+
+        if not AXObject.is_valid(obj):
+            return ""
+
+        try:
+            # Added in Atspi 2.52.
+            text = Atspi.Accessible.get_help_text(obj)
+        except Exception:
+            # This is for prototyping in the meantime.
+            text = AXObject.get_attribute(obj, "helptext") or ""
+
+        return text
+
     @staticmethod
     def get_child_count(obj):
         """Returns the child count of obj"""
@@ -747,6 +853,10 @@ class AXObject:
             return
 
         child_count = AXObject.get_child_count(obj)
+        if child_count > 500:
+            tokens = ["AXObject:", obj, "has more than 500 children"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True, True)
+
         for index in range(child_count):
             child = AXObject.get_child(obj, index)
             if child is not None and (pred is None or pred(child)):
@@ -886,91 +996,6 @@ class AXObject:
 
         return ", ".join(map(as_string, AXObject.get_state_set(obj).get_states()))
 
-    @staticmethod
-    def get_relations(obj):
-        """Returns the list of Atspi.Relation objects associated with obj"""
-
-        if not AXObject.is_valid(obj):
-            return []
-
-        try:
-            relations = Atspi.Accessible.get_relation_set(obj)
-        except Exception as error:
-            msg = f"AXObject: Exception in get_relations: {error}"
-            AXObject.handle_error(obj, error, msg)
-            return []
-
-        return relations
-
-    @staticmethod
-    def get_relation(obj, relation_type):
-        """Returns the specified Atspi.Relation for obj"""
-
-        if not AXObject.is_valid(obj):
-            return None
-
-        for relation in AXObject.get_relations(obj):
-            if relation and relation.get_relation_type() == relation_type:
-                return relation
-
-        return None
-
-    @staticmethod
-    def has_relation(obj, relation_type):
-        """Returns true if obj has the specified relation type"""
-
-        if not AXObject.is_valid(obj):
-            return False
-
-        return AXObject.get_relation(obj, relation_type) is not None
-
-    @staticmethod
-    def get_relation_targets(obj, relation_type, pred=None):
-        """Returns the list of targets with the specified relation type to obj.
-        If pred is provided, a target will only be included if pred is true."""
-
-        if not AXObject.is_valid(obj):
-            return []
-
-        relation = AXObject.get_relation(obj, relation_type)
-        if relation is None:
-            return []
-
-        targets = set()
-        for i in range(relation.get_n_targets()):
-            target = relation.get_target(i)
-            if pred is None or pred(target):
-                targets.add(target)
-
-        # We want to avoid self-referential relationships.
-        type_includes_object = [Atspi.RelationType.MEMBER_OF]
-        if relation_type not in type_includes_object and obj in targets:
-            tokens = ["AXObject: ", obj, "is in its own", relation_type, "target list"]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
-            targets.remove(obj)
-
-        return list(targets)
-
-    @staticmethod
-    def relations_as_string(obj):
-        """Returns the relations associated with obj as a string"""
-
-        if not AXObject.is_valid(obj):
-            return ""
-
-        def as_string(relations):
-            return relations.value_name[15:].replace("_", "-").lower()
-
-        results = []
-        for rel in AXObject.get_relations(obj):
-            type_string = as_string(rel.get_relation_type())
-            targets = AXObject.get_relation_targets(obj, rel.get_relation_type())
-            target_string = ",".join(map(str, targets))
-            results.append(f"{type_string}: {target_string}")
-
-        return "; ".join(results)
-
-
     @staticmethod
     def find_real_app_and_window_for(obj, app=None):
         """Work around for window events coming from mutter-x11-frames."""
@@ -1107,23 +1132,24 @@ class AXObject:
         return string
 
     @staticmethod
-    def clear_cache(obj, recursive=False):
+    def clear_cache(obj, recursive=False, reason=""):
         """Clears the Atspi cached information associated with obj"""
 
-        if not AXObject.is_valid(obj):
+        if obj is None:
             return
 
+        tokens = ["AXObject: Clearing AT-SPI cache on", obj, f"Recursive: {recursive}."]
+        if reason:
+            tokens.append(f" Reason: {reason}")
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+
         if not recursive:
             try:
                 Atspi.Accessible.clear_cache_single(obj)
-            except Exception:
-                # This is new API, added in 2.49.1. So log success rather than
-                # (likely) failure for now.
-                pass
-            else:
-                msg = "AXObject: clear_cache_single succeeded."
+            except Exception as error:
+                msg = f"AXObject: Exception in clear_cache_single: {error}"
                 debug.printMessage(debug.LEVEL_INFO, msg, True)
-                return
+            return
 
         try:
             Atspi.Accessible.clear_cache(obj)
@@ -1170,12 +1196,17 @@ class AXObject:
         return False
 
     @staticmethod
-    def get_attributes_dict(obj):
+    def get_attributes_dict(obj, use_cache=True):
         """Returns the object attributes of obj as a dictionary."""
 
         if not AXObject.is_valid(obj):
             return {}
 
+        if use_cache:
+            attributes = AXObject.OBJECT_ATTRIBUTES.get(hash(obj))
+            if attributes:
+                return attributes
+
         try:
             attributes = Atspi.Accessible.get_attributes(obj)
         except Exception as error:
@@ -1186,16 +1217,17 @@ class AXObject:
         if attributes is None:
             return {}
 
+        AXObject.OBJECT_ATTRIBUTES[hash(obj)] = attributes
         return attributes
 
     @staticmethod
-    def get_attribute(obj, attribute_name):
+    def get_attribute(obj, attribute_name, use_cache=True):
         """Returns the value of the specified attribute as a string."""
 
         if not AXObject.is_valid(obj):
             return ""
 
-        attributes = AXObject.get_attributes_dict(obj)
+        attributes = AXObject.get_attributes_dict(obj, use_cache)
         return attributes.get(attribute_name, "")
 
     @staticmethod
@@ -1294,6 +1326,9 @@ class AXObject:
             AXObject.handle_error(obj, error, msg)
             return ""
 
+        # GTK4 does this.
+        if keybinding == "<VoidSymbol>":
+            return ""
         return keybinding
 
     @staticmethod
@@ -1359,4 +1394,27 @@ class AXObject:
 
         return "; ".join(results)
 
+    @staticmethod
+    def grab_focus(obj):
+        """Attempts to grab focus on obj. Returns true if successful."""
+
+        if not AXObject.supports_component(obj):
+            return False
+
+        try:
+            result = Atspi.Component.grab_focus(obj)
+        except Exception as error:
+            msg = f"AXObject: Exception in grab_focus: {error}"
+            AXObject.handle_error(obj, error, msg)
+            return False
+
+        if debug.LEVEL_INFO < debug.debugLevel:
+            return result
+
+        if result and not AXObject.has_state(obj, Atspi.StateType.FOCUSED):
+            tokens = ["AXObject:", obj, "lacks focused state after focus grab"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+
+        return result
+
 AXObject.start_cache_clearing_thread()
