@@ -18,6 +18,10 @@
 # Free Software Foundation, Inc., Franklin Street, Fifth Floor,
 # Boston MA  02110-1301 USA.
 
+# pylint: disable=wrong-import-position
+# pylint: disable=too-many-public-methods
+# pylint: disable=too-many-lines
+
 """
 Utilities for finding all objects that meet a certain criteria.
 These utilities are app-type- and toolkit-agnostic. Utilities that might have
@@ -44,6 +48,7 @@ from gi.repository import Atspi
 from . import debug
 from .ax_collection import AXCollection
 from .ax_object import AXObject
+from .ax_utilities_relation import AXUtilitiesRelation
 from .ax_utilities_role import AXUtilitiesRole
 from .ax_utilities_state import AXUtilitiesState
 
@@ -945,10 +950,14 @@ class AXUtilitiesCollection:
         return matches
 
     @staticmethod
-    def find_all_lists(root, pred=None):
+    def find_all_lists(root, pred=None, include_description_lists=False, include_tab_lists=False):
         """Returns all descendants of root with the list role"""
 
         roles = [Atspi.Role.LIST]
+        if include_description_lists:
+            roles.append(Atspi.Role.DESCRIPTION_LIST)
+        if include_tab_lists:
+            roles.append(Atspi.Role.PAGE_TAB_LIST)
         return AXUtilitiesCollection.find_all_with_role(root, roles, pred)
 
     @staticmethod
@@ -959,10 +968,14 @@ class AXUtilitiesCollection:
         return AXUtilitiesCollection.find_all_with_role(root, roles, pred)
 
     @staticmethod
-    def find_all_list_items(root, pred=None):
+    def find_all_list_items(root, pred=None, include_description_terms=False, include_tabs=False):
         """Returns all descendants of root with the list item role"""
 
         roles = [Atspi.Role.LIST_ITEM]
+        if include_description_terms:
+            roles.append(Atspi.Role.DESCRIPTION_TERM)
+        if include_tabs:
+            roles.append(Atspi.Role.PAGE_TAB)
         return AXUtilitiesCollection.find_all_with_role(root, roles, pred)
 
     @staticmethod
@@ -1580,7 +1593,7 @@ class AXUtilitiesCollection:
         """Returns all the descendants of root that have a label role, but no relations"""
 
         def _pred(obj):
-            if AXObject.get_relations(obj):
+            if not AXUtilitiesRelation.object_is_unrelated(obj):
                 return False
             if pred is not None:
                 return pred(obj)
