@@ -42,18 +42,19 @@ from . import brltablenames
 from . import cmdnames
 from . import debug
 from . import logger
-from . import orca_state
+from . import script_manager
 from . import settings
 from . import settings_manager
 
 from .ax_event_synthesizer import AXEventSynthesizer
+from .ax_hypertext import AXHypertext
 from .ax_object import AXObject
+from .ax_text import AXText
 from .orca_platform import tablesdir
 
 _logger = logger.getLogger()
 log = _logger.newLog("braille")
 _monitor = None
-_settingsManager = settings_manager.getManager()
 
 try:
     msg = "BRAILLE: About to import brlapi."
@@ -101,16 +102,6 @@ except Exception:
     settings.enableBrailleMonitor = False
 
 
-# brlapi keys which are not allowed to interrupt speech:
-#
-dontInteruptSpeechKeys = []
-if _brlAPIAvailable:
-    dontInteruptSpeechKeys = [ \
-        brlapi.KEY_CMD_HWINLT, brlapi.KEY_CMD_HWINRT, \
-        brlapi.KEY_CMD_FWINLT, brlapi.KEY_CMD_FWINRT, \
-        brlapi.KEY_CMD_FWINLTSKIP, brlapi.KEY_CMD_FWINRTSKIP, \
-        brlapi.KEY_CMD_LNUP, brlapi.KEY_CMD_LNDN]
-
 # Common names for most used BrlTTY commands, to be shown in the GUI:
 # ATM, the ones used in default.py are:
 #
@@ -198,6 +189,14 @@ _saved = None
 #
 idle = False
 
+# BRLAPI priority levels if Orca should have idle, normal or high priority
+BRLAPI_PRIORITY_IDLE = 0
+BRLAPI_PRIORITY_DEFAULT = 50
+BRLAPI_PRIORITY_HIGH = 70
+
+# Saved BRLAPI priority
+brlapi_priority = BRLAPI_PRIORITY_DEFAULT
+
 # Translators: These are the braille translation table names for different
 # languages. You could read about braille tables at:
 # http://en.wikipedia.org/wiki/Braille
@@ -305,19 +304,6 @@ if louis:
     tokens = ["BRAILLE: Default contraction table is:", _defaultContractionTable]
     debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
-def _printBrailleEvent(level, command):
-    """Prints out a Braille event.  The given level may be overridden
-    if the eventDebugLevel (see debug.setEventDebugLevel) is greater in
-    debug.py.
-
-    Arguments:
-    - command: the BrlAPI command for the key that was pressed.
-    """
-
-    debug.printInputEvent(
-        level,
-        f"BRAILLE EVENT: {repr(command)}")
-
 class Region:
     """A Braille region to be displayed on the display.  The width of
     each region is determined by its string.
@@ -362,23 +348,24 @@ class Region:
                     )
                     debug.printMessage(debug.LEVEL_INFO, msg, True)
                 else:
-                    msg = (
-                        f"BRAILLE: Not contracting '{string}' "
-                        f"due to problem with liblouis."
-                    )
-                    debug.printMessage(debug.LEVEL_INFO, msg, True)
+                    tokens = ["BRAILLE: Not contracting '", string,
+                              "' due to problem with liblouis."]
+                    debug.printTokens(debug.LEVEL_WARNING, tokens, True)
+
             self.string = self.rawLine
             self.cursorOffset = cursorOffset
 
     def __str__(self):
-        return "Region: '%s', %d" % (self.string, self.cursorOffset)
+        return f"REGION: '{self.string}', cursor offset:{self.cursorOffset}"
 
-    def processRoutingKey(self, offset):
+    def process_routing_key(self, offset):
         """Processes a cursor routing key press on this Component.  The offset
         is 0-based, where 0 represents the leftmost character of string
         associated with this region.  Note that the zeroeth character may have
         been scrolled off the display."""
-        pass
+
+        msg = f"BRAILLE REGION: Process routing key. Offset: {offset}"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     def getAttributeMask(self, getLinkMask=True):
         """Creates a string which can be used as the attrOr field of brltty's
@@ -454,7 +441,7 @@ class Region:
 
         return offset
 
-    def setContractedBraille(self, contracted):
+    def set_contracted_braille(self, contracted):
         if contracted:
             self.contractionTable = settings.brailleContractionTable or _defaultContractionTable
             self.contractRegion()
@@ -507,7 +494,7 @@ class Component(Region):
         self.accessible = accessible
 
     def __str__(self):
-        return "Component: '%s', %d" % (self.string, self.cursorOffset)
+        return f"COMPONENT: '{self.string}', cursor offset:{self.cursorOffset}"
 
     def getCaretOffset(self, offset):
         """Returns the caret position of the given offset if the object
@@ -518,18 +505,18 @@ class Component(Region):
         """
         return -1
 
-    def processRoutingKey(self, offset):
+    def process_routing_key(self, offset):
         """Processes a cursor routing key press on this Component.  The offset
         is 0-based, where 0 represents the leftmost character of string
         associated with this region.  Note that the zeroeth character may have
         been scrolled off the display."""
 
-        if orca_state.activeScript and orca_state.activeScript.utilities.\
-           grabFocusBeforeRouting(self.accessible, offset):
-            try:
-                self.accessible.queryComponent().grabFocus()
-            except Exception:
-                pass
+        msg = f"BRAILLE COMPONENT: Process routing key. Offset: {offset}"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+
+        script = script_manager.get_manager().get_active_script()
+        if script and script.utilities.grabFocusBeforeRouting(self.accessible, offset):
+            AXObject.grab_focus(self.accessible)
 
         if AXObject.do_action(self.accessible, 0):
             return
@@ -558,7 +545,7 @@ class Link(Component):
         Component.__init__(self, accessible, string, cursorOffset, '', True)
 
     def __str__(self):
-        return "Link: '%s', %d" % (self.string, self.cursorOffset)
+        return f"LINK: '{self.string}', cursor offset:{self.cursorOffset}"
 
     def getAttributeMask(self, getLinkMask=True):
         """Creates a string which can be used as the attrOr field of brltty's
@@ -585,7 +572,7 @@ class Text(Region):
     as bugzilla bug 319754.]]]"""
 
     def __init__(self, accessible, label="", eol="",
-                 startOffset=None, endOffset=None):
+                 startOffset=None, endOffset=None, caretOffset=None):
         """Creates a new Text region.
 
         Arguments:
@@ -593,15 +580,24 @@ class Text(Region):
         - label: an optional label to display
         """
 
+        tokens = ["BRAILLE: Creating text region for", accessible,
+                  f"label:'{label}', offsets: {startOffset}-{endOffset}, caret: {caretOffset}"]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+
         self.accessible = accessible
-        if orca_state.activeScript and self.accessible:
-            [string, self.caretOffset, self.lineOffset] = \
-                 orca_state.activeScript.getTextLineAtCaret(
-                     self.accessible, startOffset=startOffset, endOffset=endOffset)
-        else:
-            string = ""
-            self.caretOffset = 0
-            self.lineOffset = 0
+        string = ""
+        self.caretOffset = 0
+        self.lineOffset = 0
+        if self.accessible:
+            if caretOffset is not None:
+                self.caretOffset = caretOffset
+            else:
+                self.caretOffset = AXText.get_caret_offset(self.accessible)
+            if startOffset is not None:
+                self.caretOffset = max(startOffset, self.caretOffset)
+            string, self.lineOffset = AXText.get_line_at_offset(
+                self.accessible, self.caretOffset)[0:2]
+            string = string.replace("\ufffc", " ")
 
         try:
             endOffset = endOffset - self.lineOffset
@@ -643,7 +639,10 @@ class Text(Region):
             self.string += ' '
 
     def __str__(self):
-        return "Text: '%s', %d" % (self.string, self.cursorOffset)
+        return (
+            f"TEXT: '{self.string}', cursor offset:{self.cursorOffset} "
+            f"start offset:{self.startOffset}, line offset:{self.lineOffset}"
+        )
 
     def repositionCursor(self):
         """Attempts to reposition the cursor in response to a new
@@ -655,9 +654,8 @@ class Text(Region):
         if not _regionWithFocus:
             return False
 
-        [string, caretOffset, lineOffset] = \
-                 orca_state.activeScript.getTextLineAtCaret(self.accessible)
-
+        string, lineOffset = AXText.get_line_at_offset(self.accessible)[0:2]
+        caretOffset = AXText.get_caret_offset(self.accessible)
         cursorOffset = min(caretOffset - lineOffset, len(string))
 
         if lineOffset != self.lineOffset:
@@ -690,20 +688,23 @@ class Text(Region):
 
         return min(self.lineOffset + offset, self._maxCaretOffset)
 
-    def processRoutingKey(self, offset):
+    def process_routing_key(self, offset):
         """Processes a cursor routing key press on this Component.  The offset
         is 0-based, where 0 represents the leftmost character of text
         associated with this region.  Note that the zeroeth character may have
         been scrolled off the display.
         """
 
+        msg = f"BRAILLE TEXT: Process routing key. Offset: {offset}"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+
         caretOffset = self.getCaretOffset(offset)
 
         if caretOffset < 0:
             return
 
-        orca_state.activeScript.utilities.setCaretOffset(
-            self.accessible, caretOffset)
+        script = script_manager.get_manager().get_active_script()
+        script.utilities.setCaretOffset(self.accessible, caretOffset)
 
     def getAttributeMask(self, getLinkMask=True):
         """Creates a string which can be used as the attrOr field of brltty's
@@ -717,10 +718,8 @@ class Text(Region):
           unreasonable amount of time (AKA Gecko).
         """
 
-        try:
-            self.accessible.queryText()
-        except NotImplementedError:
-            return ''
+        if AXText.is_whitespace_or_empty(self.accessible):
+            return ""
 
         # Start with an empty mask.
         #
@@ -731,29 +730,21 @@ class Text(Region):
         attrIndicator = settings.textAttributesBrailleIndicator
         selIndicator = settings.brailleSelectorIndicator
         linkIndicator = settings.brailleLinkIndicator
-        script = orca_state.activeScript
+        script = script_manager.get_manager().get_active_script()
         if script is None:
             msg = "BRAILLE: Cannot get attribute mask without active script."
             debug.printMessage(debug.LEVEL_INFO, msg, True)
             return ""
 
         if getLinkMask and linkIndicator != settings.BRAILLE_UNDERLINE_NONE:
-            try:
-                hyperText = self.accessible.queryHypertext()
-                nLinks = hyperText.getNLinks()
-            except Exception:
-                nLinks = 0
-
-            n = 0
-            while n < nLinks:
-                link = hyperText.getLink(n)
-                if self.lineOffset <= link.startIndex:
-                    for i in range(link.startIndex, link.endIndex):
-                        try:
-                            regionMask[i] |= linkIndicator
-                        except Exception:
-                            pass
-                n += 1
+            links = AXHypertext.get_all_links(self.accessible)
+            for link in links:
+                startOffset = AXHypertext.get_link_start_offset(link)
+                endOffset = AXHypertext.get_link_end_offset(link)
+                maskStart = max(startOffset - self.lineOffset, 0)
+                maskEnd = min(endOffset - self.lineOffset, stringLength)
+                for i in range(maskStart, maskEnd):
+                  regionMask[i] |= linkIndicator
 
         if attrIndicator:
             keys, enabledAttributes = script.utilities.stringToKeysAndDict(
@@ -762,8 +753,7 @@ class Text(Region):
             offset = self.lineOffset
             while offset < lineEndOffset:
                 attributes, startOffset, endOffset = \
-                    script.utilities.textAttributes(self.accessible,
-                                                    offset, True)
+                    AXText.get_text_attributes_at_offset(self.accessible, offset)
                 if endOffset <= offset:
                     break
                 mask = settings.BRAILLE_UNDERLINE_NONE
@@ -780,7 +770,7 @@ class Text(Region):
                         regionMask[i] |= attrIndicator
 
         if selIndicator:
-            selections = script.utilities.allTextSelections(self.accessible)
+            selections = AXText.get_selected_ranges(self.accessible)
             for startOffset, endOffset in selections:
                 maskStart = max(startOffset - self.lineOffset, 0)
                 maskEnd = min(endOffset - self.lineOffset, stringLength)
@@ -823,8 +813,8 @@ class Text(Region):
         offset -= len(self.label)
         return offset
 
-    def setContractedBraille(self, contracted):
-        Region.setContractedBraille(self, contracted)
+    def set_contracted_braille(self, contracted):
+        Region.set_contracted_braille(self, contracted)
         if not contracted:
             self.string += self.eol
 
@@ -886,15 +876,18 @@ class ReviewText(Region):
 
         return self.lineOffset + offset
 
-    def processRoutingKey(self, offset):
+    def process_routing_key(self, offset):
         """Processes a cursor routing key press on this Component.  The offset
         is 0-based, where 0 represents the leftmost character of text
         associated with this region.  Note that the zeroeth character may have
         been scrolled off the display."""
 
+        msg = f"BRAILLE REVIEW TEXT: Process routing key. Offset: {offset}"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+
         caretOffset = self.getCaretOffset(offset)
-        orca_state.activeScript.utilities.setCaretOffset(
-            self.accessible, caretOffset)
+        script = script_manager.get_manager().get_active_script()
+        script.utilities.setCaretOffset(self.accessible, caretOffset)
 
 class Line:
     """A horizontal line on the display.  Each Line is composed of a sequential
@@ -928,6 +921,13 @@ class Line:
         Returns [string, offsetIndex, attributeMask, ranges]
         """
 
+        # TODO: The way words are being combined here can result in incorrect range groupings.
+        # For instance, if we generate the full ancestry of a multiline text object and the
+        # line begins with whitespace, we'll wind up with a single range that contains the
+        # last word of the ancestor followed by the whitespace and the first word, e.g.
+        # "frame      Hello". We probably should not be creating a single string which we then
+        # split into words.
+
         string = ""
         focusOffset = -1
         attributeMask = ""
@@ -1000,19 +1000,22 @@ class Line:
         else:
             return [foundRegion, offset - pos]
 
-    def processRoutingKey(self, offset):
+    def process_routing_key(self, offset):
         """Processes a cursor routing key press on this Component.  The offset
         is 0-based, where 0 represents the leftmost character of string
         associated with this line.  Note that the zeroeth character may have
         been scrolled off the display."""
 
+        msg = f"BRAILLE LINE: Process routing key. Offset: {offset}"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+
         [region, regionOffset] = self.getRegionAtOffset(offset)
         if region:
-            region.processRoutingKey(regionOffset)
+            region.process_routing_key(regionOffset)
 
-    def setContractedBraille(self, contracted):
+    def set_contracted_braille(self, contracted):
         for region in self.regions:
-            region.setContractedBraille(contracted)
+            region.set_contracted_braille(contracted)
 
 def getRegionAtCell(cell):
     """Given a 1-based cell offset, return the braille region
@@ -1151,7 +1154,7 @@ def _idleBraille():
         try:
             msg = "BRAILLE: Attempting to idle braille."
             debug.printMessage(debug.LEVEL_INFO, msg, True)
-            _brlAPI.setParameter(brlapi.PARAM_CLIENT_PRIORITY, 0, False, 0)
+            _brlAPI.setParameter(brlapi.PARAM_CLIENT_PRIORITY, 0, False, BRLAPI_PRIORITY_IDLE)
             idle = True
         except Exception:
             msg = "BRAILLE: Idling braille failled. This requires BrlAPI >= 0.8."
@@ -1200,7 +1203,7 @@ def _enableBraille():
                 # Restore default priority
                 msg = "BRAILLE: Attempting to de-idle braille."
                 debug.printMessage(debug.LEVEL_INFO, msg, True)
-                _brlAPI.setParameter(brlapi.PARAM_CLIENT_PRIORITY, 0, False, 50)
+                _brlAPI.setParameter(brlapi.PARAM_CLIENT_PRIORITY, 0, False, brlapi_priority)
                 idle = False
             except Exception:
                 msg = "BRAILLE: could not restore priority"
@@ -1222,7 +1225,7 @@ def disableBraille():
         msg = "BRAILLE: BrlApi running and not idle."
         debug.printMessage(debug.LEVEL_INFO, msg, True)
 
-        if not _idleBraille() and not _settingsManager.getSetting('enableBraille'):
+        if not _idleBraille() and not settings_manager.get_manager().get_setting('enableBraille'):
             # BrlAPI before 0.8 and we really want to shut down
             msg = "BRAILLE: could not go idle, completely shut down"
             debug.printMessage(debug.LEVEL_INFO, msg, True)
@@ -1234,7 +1237,7 @@ def checkBrailleSetting():
     msg = "BRAILLE: Checking braille setting."
     debug.printMessage(debug.LEVEL_INFO, msg, True)
 
-    if not _settingsManager.getSetting('enableBraille'):
+    if not settings_manager.get_manager().get_setting('enableBraille'):
         disableBraille()
 
 def refresh(panToCursor=True, targetCursorCell=0, getLinkMask=True, stopFlash=True):
@@ -1274,8 +1277,8 @@ def refresh(panToCursor=True, targetCursorCell=0, getLinkMask=True, stopFlash=Tr
         killFlash(restoreSaved=False)
 
     # TODO - JD: This should be taken care of in orca.py.
-    if not _settingsManager.getSetting('enableBraille') \
-       and not _settingsManager.getSetting('enableBrailleMonitor'):
+    if not settings_manager.get_manager().get_setting('enableBraille') \
+       and not settings_manager.get_manager().get_setting('enableBrailleMonitor'):
         if _brlAPIRunning:
             msg = "BRAILLE: FIXME - Braille disabled, but not properly shut down."
             debug.printMessage(debug.LEVEL_INFO, msg, True)
@@ -1423,10 +1426,10 @@ def refresh(panToCursor=True, targetCursorCell=0, getLinkMask=True, stopFlash=Tr
 
     submask += '\x00' * (len(substring) - len(submask))
 
-    if _settingsManager.getSetting('enableBraille'):
+    if settings_manager.get_manager().get_setting('enableBraille'):
         _enableBraille()
 
-    if _settingsManager.getSetting('enableBraille') and _brlAPIRunning:
+    if settings_manager.get_manager().get_setting('enableBraille') and _brlAPIRunning:
         writeStruct = brlapi.WriteStruct()
         writeStruct.regionBegin = 1
         writeStruct.regionSize = len(substring)
@@ -1502,12 +1505,17 @@ def _flashCallback():
 
     if _flashEventSourceId:
         (_lines, _regionWithFocus, viewport, flashTime) = _saved
+        msg = "BRAILLE: Flash message callback"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         refresh(panToCursor=False, stopFlash=False)
         _flashEventSourceId = 0
 
     return False
 
 def killFlash(restoreSaved=True):
+    msg = "BRAILLE: Kill flash message"
+    debug.printMessage(debug.LEVEL_INFO, msg, True, True)
+
     global _flashEventSourceId
     global _lines
     global _regionWithFocus
@@ -1542,6 +1550,9 @@ def _initFlash(flashTime):
     global _saved
     global _flashEventSourceId
 
+    msg = f"BRAILLE: Initializing flash: Source ID: {_flashEventSourceId}"
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
+
     if _flashEventSourceId:
         if _flashEventSourceId > 0:
             GLib.source_remove(_flashEventSourceId)
@@ -1557,7 +1568,7 @@ def _initFlash(flashTime):
 def displayRegions(regionInfo, flashTime=0):
     """Displays a list of regions on a single line, setting focus to the
        specified region.  The regionInfo parameter is something that is
-       typically returned by a call to braille_generator.generateBraille.
+       typically returned by a call to braille_generator.generate_braille.
 
     Arguments:
     - regionInfo: a list where the first element is a list of regions
@@ -1596,6 +1607,9 @@ def displayMessage(message, cursor=-1, flashTime=0):
                   comes along or the user presses a cursor routing key.
     """
 
+    msg = f"BRAILLE: Display message: '{message}' (flashTime: {flashTime})"
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
+
     _initFlash(flashTime)
     clear()
     region = Region(message, cursor)
@@ -1607,9 +1621,9 @@ def displayKeyEvent(event):
     """Displays a KeyboardEvent. Typically reserved for locking keys like
     Caps Lock and Num Lock."""
 
-    lockingStateString = event.getLockingStateString()
+    lockingStateString = event.get_locking_state_string()
     if lockingStateString:
-        keyname = event.getKeyName()
+        keyname = event.get_key_name()
         msg = f"{keyname} {lockingStateString}"
         displayMessage(msg, flashTime=settings.brailleFlashTime)
 
@@ -1647,53 +1661,53 @@ def _getRangeForOffset(offset):
 
     return [0, 0]
 
-def panLeft(panAmount=0):
+def panLeft(pan_amount=0):
     """Pans the display to the left, limiting the pan to the beginning
     of the line being displayed.
 
     Arguments:
-    - panAmount: the amount to pan.  A value of 0 means the entire
+    - pan_amount: the amount to pan.  A value of 0 means the entire
                  width of the physical display.
 
     Returns True if a pan actually happened.
     """
 
     oldX = viewport[0]
-    if panAmount == 0:
+    if pan_amount == 0:
         oldStart, oldEnd = _getRangeForOffset(oldX)
         newStart, newEnd = _getRangeForOffset(oldStart - _displaySize[0])
-        panAmount = max(0, min(oldStart - newStart, _displaySize[0]))
+        pan_amount = max(0, min(oldStart - newStart, _displaySize[0]))
 
-    viewport[0] = max(0, viewport[0] - panAmount)
-    msg = f"BRAILLE: Panning left. Amount: {panAmount} (from {oldX} to {viewport[0]})"
+    viewport[0] = max(0, viewport[0] - pan_amount)
+    msg = f"BRAILLE: Panning left. Amount: {pan_amount} (from {oldX} to {viewport[0]})"
     debug.printMessage(debug.LEVEL_INFO, msg, True)
     return oldX != viewport[0]
 
-def panRight(panAmount=0):
+def panRight(pan_amount=0):
     """Pans the display to the right, limiting the pan to the length
     of the line being displayed.
 
     Arguments:
-    - panAmount: the amount to pan.  A value of 0 means the entire
+    - pan_amount: the amount to pan.  A value of 0 means the entire
                  width of the physical display.
 
     Returns True if a pan actually happened.
     """
 
     oldX = viewport[0]
-    if panAmount == 0:
+    if pan_amount == 0:
         oldStart, oldEnd = _getRangeForOffset(oldX)
         newStart, newEnd = _getRangeForOffset(oldEnd)
-        panAmount = max(0, min(newStart - oldStart, _displaySize[0]))
+        pan_amount = max(0, min(newStart - oldStart, _displaySize[0]))
 
     if len(_lines) > 0:
         lineNum = viewport[1]
-        newX = viewport[0] + panAmount
+        newX = viewport[0] + pan_amount
         string, focusOffset, attributeMask, ranges = _lines[lineNum].getLineInfo()
         if newX < len(string):
             viewport[0] = newX
 
-    msg = f"BRAILLE: Panning right. Amount: {panAmount} (from {oldX} to {viewport[0]})"
+    msg = f"BRAILLE: Panning right. Amount: {pan_amount} (from {oldX} to {viewport[0]})"
     debug.printMessage(debug.LEVEL_INFO, msg, True)
     return oldX != viewport[0]
 
@@ -1726,7 +1740,7 @@ def returnToRegionWithFocus(inputEvent=None):
 
     return True
 
-def setContractedBraille(event):
+def set_contracted_braille(event):
     """Turns contracted braille on or off based upon the event.
 
     Arguments:
@@ -1737,10 +1751,10 @@ def setContractedBraille(event):
     settings.enableContractedBraille = \
         (event.event["flags"] & brlapi.KEY_FLG_TOGGLE_ON) != 0
     for line in _lines:
-        line.setContractedBraille(settings.enableContractedBraille)
+        line.set_contracted_braille(settings.enableContractedBraille)
     refresh()
 
-def processRoutingKey(event):
+def process_routing_key(event):
     """Processes a cursor routing key event.
 
     Arguments:
@@ -1748,8 +1762,9 @@ def processRoutingKey(event):
     the dictionary form of the expanded BrlAPI event.
     """
 
-    # If a message is being flashed, we'll use a routing key to dismiss it.
-    #
+    msg = f"BRAILLE: Process routing key. Source ID: {_flashEventSourceId}"
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
+
     if _flashEventSourceId:
         killFlash()
         return
@@ -1759,7 +1774,7 @@ def processRoutingKey(event):
     if len(_lines) > 0:
         cursor = cell + viewport[0]
         lineNum = viewport[1]
-        _lines[lineNum].processRoutingKey(cursor)
+        _lines[lineNum].process_routing_key(cursor)
 
     return True
 
@@ -1771,9 +1786,8 @@ def _processBrailleEvent(event):
     - event: the BrlAPI input event (expanded)
     """
 
-    _printBrailleEvent(debug.LEVEL_FINE, event)
-
-    consumed = False
+    tokens = ["BRAILLE: Processing event", event]
+    debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
     if settings.timeoutCallback and (settings.timeoutTime > 0):
         signal.signal(signal.SIGALRM, settings.timeoutCallback)
@@ -1785,9 +1799,9 @@ def _processBrailleEvent(event):
             # the command was consumed.
             #
             consumed = _callback(event)
-        except Exception:
-            debug.printMessage(debug.LEVEL_WARNING, "Issue processing event:")
-            debug.printException(debug.LEVEL_WARNING)
+        except Exception as error:
+            msg = f"WARNING: Could not process braille event: {error}"
+            debug.printMessage(debug.LEVEL_WARNING, msg, True)
             consumed = False
 
     if settings.timeoutCallback and (settings.timeoutTime > 0):
@@ -1801,9 +1815,9 @@ def _brlAPIKeyReader(source, condition):
     """
     try:
         key = _brlAPI.readKey(False)
-    except Exception:
-        debug.printMessage(debug.LEVEL_WARNING, "BrlTTY seems to have disappeared:")
-        debug.printException(debug.LEVEL_WARNING)
+    except Exception as error:
+        msg = f"WARNING: Could not read BrlApi key: {error}"
+        debug.printMessage(debug.LEVEL_WARNING, msg, True)
         shutdown()
         return
     if key:
@@ -1848,6 +1862,37 @@ def setupKeyRanges(keys):
     msg = "BRAILLE: Key ranges set up."
     debug.printMessage(debug.LEVEL_INFO, msg, True)
 
+def setBrlapiPriority(level=BRLAPI_PRIORITY_DEFAULT):
+    """Set BRLAPI priority
+
+    Arguments:
+    -level: the priority level to apply, default to braille.PRIORITY_DEFAULT
+    """
+
+    global idle, brlapi_priority
+
+    if not _brlAPIAvailable or not _brlAPIRunning \
+       or not settings_manager.get_manager().get_setting('enableBraille'):
+        return
+
+    if idle:
+        msg = "BRAILLE: Braille is idle, don't change BRLAPI priority."
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        brlapi_priority = level
+        return
+
+    try:
+        tokens = ["BRAILLE: Setting priority to:", level]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+        _brlAPI.setParameter(brlapi.PARAM_CLIENT_PRIORITY, 0, False, level)
+    except Exception as error:
+        msg = f"BRAILLE: Cannot set priority: {error}"
+        debug.printMessage(debug.LEVEL_WARNING, msg, True)
+    else:
+        msg = "BRAILLE: Priority set."
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        brlapi_priority = level
+
 def init(callback=None):
     """Initializes the braille module, connecting to the BrlTTY driver.
 
@@ -1918,10 +1963,9 @@ def init(callback=None):
         msg = "BRAILLE: Initialization failed: BrlApi is not defined."
         debug.printMessage(debug.LEVEL_WARNING, msg, True)
         return False
-    except Exception:
-        msg = "BRAILLE: Initialization failed."
+    except Exception as error:
+        msg = f"WARNING: Braille initialization failed: {error}"
         debug.printMessage(debug.LEVEL_WARNING, msg, True)
-        debug.printException(debug.LEVEL_WARNING)
 
         _brlAPIRunning = False
 
