@@ -34,13 +34,15 @@ import re
 
 from . import braille
 from . import debug
-from . import orca
-from . import orca_state
+from . import focus_manager
+from . import script_manager
 from . import settings
-from .ax_event_synthesizer import AXEventSynthesizer
+from .ax_component import AXComponent
 from .ax_object import AXObject
+from .ax_text import AXText
 from .ax_utilities import AXUtilities
 
+
 EMBEDDED_OBJECT_CHARACTER = '\ufffc'
 
 class Char:
@@ -66,6 +68,11 @@ class Char:
         self.width = width
         self.height = height
 
+    def __str__(self):
+        return "CHAR: '%s' (%i-%i)" % \
+            (self.string.replace("\n", "\\n"),
+             self.startOffset,
+             self.endOffset)
 
 class Word:
     """A single chunk (word or object) of presentable information."""
@@ -104,24 +111,11 @@ class Word:
         if attr != "chars":
             return super().__getattribute__(attr)
 
-        # TODO - JD: For now, don't fake character and word extents.
-        # The main goal is to improve reviewability.
-        extents = self.x, self.y, self.width, self.height
-
-        try:
-            text = self.zone.accessible.queryText()
-        except Exception:
-            text = None
-
         chars = []
         for i, char in enumerate(self.string):
             start = i + self.startOffset
-            if text:
-                try:
-                    extents = text.getRangeExtents(start, start+1, Atspi.CoordType.SCREEN)
-                except Exception as error:
-                    tokens = ["FLAT REVIEW: Exception in getRangeExtents:", error]
-                    debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            rect1 = AXText.get_character_rect(self.zone.accessible, start)
+            extents = rect1.x, rect1.y, rect1.width, rect1.height
             chars.append(Char(self, i, start, char, *extents))
 
         return chars
@@ -276,7 +270,6 @@ class TextZone(Zone):
 
         self.startOffset = startOffset
         self.endOffset = self.startOffset + len(string)
-        self._itext = self.accessible.queryText()
 
     def __getattribute__(self, attr):
         """To ensure we update the content."""
@@ -284,11 +277,12 @@ class TextZone(Zone):
         if attr not in ["words", "string"]:
             return super().__getattribute__(attr)
 
-        string = self._itext.getText(self.startOffset, self.endOffset)
+        string = AXText.get_substring(self.accessible, self.startOffset, self.endOffset)
         words = []
         for i, word in enumerate(re.finditer(self.WORDS_RE, string)):
             start, end = map(lambda x: x + self.startOffset, word.span())
-            extents = self._itext.getRangeExtents(start, end, Atspi.CoordType.SCREEN)
+            rect = AXText.get_range_rect(self.accessible, start, end)
+            extents = rect.x, rect.y, rect.width, rect.height
             words.append(Word(self, i, start, word.group(), *extents))
 
         self._string = string
@@ -298,11 +292,10 @@ class TextZone(Zone):
     def hasCaret(self):
         """Returns True if this Zone contains the caret."""
 
-        offset = self._itext.caretOffset
-        if self.startOffset <= offset < self.endOffset:
+        if self.startOffset <= AXText.get_caret_offset(self.accessible) < self.endOffset:
             return True
 
-        return self.endOffset == self._itext.characterCount
+        return self.endOffset == AXText.get_character_count(self.accessible)
 
     def wordWithCaret(self):
         """Returns the Word and relative offset with the caret."""
@@ -310,7 +303,7 @@ class TextZone(Zone):
         if not self.hasCaret():
             return None, -1
 
-        return self.getWordAtOffset(self._itext.caretOffset)
+        return self.getWordAtOffset(AXText.get_caret_offset(self.accessible))
 
 
 class StateZone(Zone):
@@ -325,12 +318,13 @@ class StateZone(Zone):
         if attr not in ["string", "brailleString"]:
             return super().__getattribute__(attr)
 
+        script = script_manager.get_manager().get_active_script()
         if attr == "string":
-            generator = orca_state.activeScript.speechGenerator
+            generator = script.speech_generator
         else:
-            generator = orca_state.activeScript.brailleGenerator
+            generator = script.braille_generator
 
-        result = generator.getStateIndicator(self.accessible, role=self.role)
+        result = generator.get_state_indicator(self.accessible, role=self.role)
         if result:
             return result[0]
 
@@ -349,17 +343,18 @@ class ValueZone(Zone):
         if attr not in ["string", "brailleString"]:
             return super().__getattribute__(attr)
 
+        script = script_manager.get_manager().get_active_script()
         if attr == "string":
-            generator = orca_state.activeScript.speechGenerator
+            generator = script.speech_generator
         else:
-            generator = orca_state.activeScript.brailleGenerator
+            generator = script.braille_generator
 
         result = ""
 
         # TODO - JD: This cobbling together beats what we had, but the
         # generators should also be doing the assembly.
-        rolename = generator.getLocalizedRoleName(self.accessible)
-        value = generator.getValue(self.accessible)
+        rolename = generator.get_localized_role_name(self.accessible)
+        value = generator.get_value(self.accessible)
         if rolename and value:
             result = f"{rolename} {value[0]}"
 
@@ -413,13 +408,13 @@ class Line:
                 # to handle problems with Java text. See Bug 435553.
                 if isinstance(zone, TextZone) and \
                    ((AXObject.get_role(zone.accessible) in \
-                         (Atspi.Role.TEXT,  
+                         (Atspi.Role.TEXT,
                           Atspi.Role.PASSWORD_TEXT,
                           Atspi.Role.TERMINAL)) or \
-                    # [[[TODO: Eitan - HACK: 
+                    # [[[TODO: Eitan - HACK:
                     # This is just to get FF3 cursor key routing support.
                     # We really should not be determining all this stuff here,
-                    # it should be in the scripts. 
+                    # it should be in the scripts.
                     # Same applies to roles above.]]]
                     (AXObject.get_role(zone.accessible) in \
                          (Atspi.Role.PARAGRAPH,
@@ -490,9 +485,9 @@ class Context:
         self.targetCharInfo = None
         self.focusZone = None
         self.container = None
-        self.focusObj = orca.getActiveModeAndObjectOfInterest()[1] or orca_state.locusOfFocus
+        self.focusObj = focus_manager.get_manager().get_locus_of_focus()
         self.topLevel = None
-        self.bounds = 0, 0, 0, 0
+        self.bounds = Atspi.Rect()
 
         frame, dialog = script.utilities.frameAndDialog(self.focusObj)
         if root is not None:
@@ -504,12 +499,7 @@ class Context:
         tokens = ["FLAT REVIEW: Frame:", frame, "Dialog:", dialog, ". Top level:", self.topLevel]
         debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
-        try:
-            component = self.topLevel.queryComponent()
-            self.bounds = component.getExtents(Atspi.CoordType.SCREEN)
-        except Exception:
-            tokens = ["ERROR: Exception getting extents of", self.topLevel]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+        self.bounds = AXComponent.get_rect(self.topLevel)
 
         containerRoles = [Atspi.Role.MENU]
 
@@ -562,31 +552,14 @@ class Context:
         substrings = [(*m.span(), m.group(0))  for m in re.finditer(r"[^\ufffc]+", string)]
         substrings = list(map(lambda x: (x[0] + startOffset, x[1] + startOffset, x[2]), substrings))
         for (start, end, substring) in substrings:
-            extents = accessible.queryText().getRangeExtents(start, end, Atspi.CoordType.SCREEN)
-            if self.script.utilities.containsRegion(extents, cliprect):
-                clipping = self.script.utilities.intersection(extents, cliprect)
+            rect = AXText.get_range_rect(accessible, start, end)
+            intersection = AXComponent.get_rect_intersection(rect, cliprect)
+            if not AXComponent.is_empty_rect(intersection):
+                clipping = intersection.x, intersection.y, intersection.width, intersection.height
                 zones.append(TextZone(accessible, start, substring, *clipping))
 
         return zones
 
-    def _getLines(self, accessible, startOffset, endOffset):
-        # TODO - JD: Move this into the script utilities so we can better handle
-        # app and toolkit quirks and also reuse this (e.g. for SayAll).
-        try:
-            text = accessible.queryText()
-        except NotImplementedError:
-            return []
-
-        lines = []
-        offset = startOffset
-        while offset < min(endOffset, text.characterCount):
-            result = text.getTextAtOffset(offset, Atspi.TextBoundaryType.LINE_START)
-            if result[0] and result not in lines:
-                lines.append(result)
-            offset = max(result[2], offset + 1)
-
-        return lines
-
     def getZonesFromText(self, accessible, cliprect):
         """Gets a list of Zones from an object that implements the
         AccessibleText specialization.
@@ -602,50 +575,28 @@ class Context:
             return []
 
         zones = []
-        text = accessible.queryText()
 
-        # TODO - JD: This is here temporarily whilst I sort out the rest
-        # of the text-related mess.
-        if AXObject.supports_editable_text(accessible) \
-           and AXUtilities.is_single_line(accessible):
-            extents = accessible.queryComponent().getExtents(0)
-            return [TextZone(accessible, 0, text.getText(0, -1), *extents)]
+        def _is_container(x):
+            return AXUtilities.is_scroll_pane(x) or AXUtilities.is_document(x)
 
-        upperMax = lowerMax = text.characterCount
-        upperMid = lowerMid = int(upperMax / 2)
-        upperMin = lowerMin = 0
-        oldMid = 0
+        container = AXObject.find_ancestor(accessible, _is_container)
+        if container:
+            rect = AXComponent.get_rect(container)
+            intersection = AXComponent.get_rect_intersection(rect, cliprect)
+            if AXComponent.is_same_rect(rect, intersection):
+                tokens = ["FLAT REVIEW: Cliprect", cliprect, "->", rect, "from", container]
+                debug.printTokens(debug.LEVEL_INFO, tokens, True)
+                cliprect = rect
 
-        # performing binary search to locate first line inside clipped area
-        while oldMid != upperMid:
-            oldMid = upperMid
-            [x, y, width, height] = text.getRangeExtents(upperMid,
-                                                         upperMid+1,
-                                                         0)
-            if y > cliprect.y:
-                upperMax = upperMid
-            else:
-                upperMin = upperMid
-            upperMid = int((upperMax - upperMin) / 2) + upperMin
+        if AXObject.supports_editable_text(accessible) and AXUtilities.is_single_line(accessible):
+            rect = AXComponent.get_rect(accessible)
+            extents = rect.x, rect.y, rect.width, rect.height
+            return [TextZone(accessible, 0, AXText.get_all_text(accessible), *extents)]
 
-        # performing binary search to locate last line inside clipped area
-        oldMid = 0
-        limit = cliprect.y+cliprect.height
-        while oldMid != lowerMid:
-            oldMid = lowerMid
-            [x, y, width, height] = text.getRangeExtents(lowerMid,
-                                                         lowerMid+1,
-                                                         0)
-            if y > limit:
-                lowerMax = lowerMid
-            else:
-                lowerMin = lowerMid
-            lowerMid = int((lowerMax - lowerMin) / 2) + lowerMin
+        tokens = ["FLAT REVIEW: Getting lines for", accessible]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
-        msg = "FLAT REVIEW: Getting lines for %s offsets %i-%i" % (accessible, upperMin, lowerMax)
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
-
-        lines = self._getLines(accessible, upperMin, lowerMax)
+        lines = AXText.get_visible_lines(accessible, cliprect)
         tokens = ["FLAT REVIEW:", len(lines), "lines found for", accessible]
         debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
@@ -662,7 +613,7 @@ class Context:
         # TODO - JD: This whole thing is pretty hacky. Either do it
         # right or nuke it.
 
-        indicatorExtents = [extents.x, extents.y, 1, extents.height]
+        indicatorExtents = [extents[0], extents[1], 1, extents[3]]
         role = AXObject.get_role(accessible)
         if role == Atspi.Role.TOGGLE_BUTTON:
             zone = StateZone(accessible, *indicatorExtents, role=role)
@@ -685,8 +636,8 @@ class Context:
 
         if len(zones) == 1 and isinstance(zones[0], TextZone):
             textZone = zones[0]
-            textToLeftEdge = textZone.x - extents.x
-            textToRightEdge = (extents.x + extents.width) - (textZone.x + textZone.width)
+            textToLeftEdge = textZone.x - extents[0]
+            textToRightEdge = (extents[0] + extents[2]) - (textZone.x + textZone.width)
             stateOnLeft = textToLeftEdge > 20
             if stateOnLeft:
                 indicatorExtents[2] = textToLeftEdge
@@ -704,17 +655,9 @@ class Context:
     def getZonesFromAccessible(self, accessible, cliprect):
         """Returns a list of Zones for the given accessible."""
 
-        try:
-            component = accessible.queryComponent()
-            extents = component.getExtents(Atspi.CoordType.SCREEN)
-        except Exception:
-            return []
-
-        try:
-            role = AXObject.get_role(accessible)
-        except Exception:
-            return []
-
+        rect = AXComponent.get_rect(accessible)
+        extents = rect.x, rect.y, rect.width, rect.height
+        role = AXObject.get_role(accessible)
         zones = self.getZonesFromText(accessible, cliprect)
         if not zones and role in [Atspi.Role.SCROLL_BAR,
                                   Atspi.Role.SLIDER,
@@ -724,11 +667,11 @@ class Context:
             string = ""
             redundant = [Atspi.Role.TABLE_ROW]
             if role not in redundant:
-                string = self.script.speechGenerator.getName(accessible, inFlatReview=True)
+                string = self.script.speech_generator.get_name(accessible, inFlatReview=True)
 
             useless = [Atspi.Role.TABLE_CELL, Atspi.Role.LABEL]
             if not string and role not in useless:
-                string = self.script.speechGenerator.getRoleName(accessible)
+                string = self.script.speech_generator.get_role_name(accessible)
             if string:
                 zones.append(Zone(accessible, string, *extents))
 
@@ -748,15 +691,10 @@ class Context:
     def setCurrentToZoneWithObject(self, obj):
         """Attempts to set the current zone to obj, if obj is in the current context."""
 
-        def _toString(x):
-            if AXObject.get_name(x):
-                return str(x)
-            return f"{x}: '{self.script.utilities.displayedText(x)}'"
-
-        msg = "FLAT REVIEW: Current %s (line: %i, zone: %i, word: %i, char: %i)" % \
-              (_toString(self.getCurrentAccessible()),
-               self.lineIndex, self.zoneIndex, self.wordIndex, self.charIndex)
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["FLAT REVIEW: Current", self.getCurrentAccessible(),
+                  f"line: {self.lineIndex}, zone: {self.zoneIndex},",
+                  f"word: {self.wordIndex}, char: {self.charIndex})"]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         zone = self._findZoneWithObject(obj)
         tokens = ["FLAT REVIEW: Zone with", obj, "is", zone]
@@ -780,10 +718,10 @@ class Context:
             debug.printMessage(debug.LEVEL_INFO, msg, True)
             return False
 
-        msg = "FLAT REVIEW: Updated %s (line: %i, zone: %i, word: %i, char: %i)" % \
-              (_toString(self.getCurrentAccessible()),
-               self.lineIndex, self.zoneIndex, self.wordIndex, self.charIndex)
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["FLAT REVIEW: Updated", self.getCurrentAccessible(),
+                  f"line: {self.lineIndex}, zone: {self.zoneIndex},",
+                  f"word: {self.wordIndex}, char: {self.charIndex})"]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
         return True
 
     def _findZoneWithObject(self, obj):
@@ -906,38 +844,6 @@ class Context:
         self.charIndex = charIndex
         self.targetCharInfo = self.getCurrent(Context.CHAR)
 
-    def _getClickPoint(self):
-        string, x, y, width, height = self.getCurrent(Context.CHAR)
-        if (x < 0 and y < 0) or (width <= 0 and height <=0):
-            return -1, -1
-
-        # Click left of center to position the caret there.
-        x = int(max(x, x + (width / 2) - 1))
-        y = int(y + height / 2)
-
-        return x, y
-
-    def routeToCurrent(self):
-        """Routes the mouse pointer to the current accessible."""
-
-        x, y = self._getClickPoint()
-        if x < 0 or y < 0:
-            return False
-
-        return AXEventSynthesizer.route_to_point(x, y)
-
-    def clickCurrent(self, button=1):
-        """Performs a mouse click on the current accessible."""
-
-        x, y = self._getClickPoint()
-        if x >= 0 and y >= 0 and AXEventSynthesizer.click_point(x, y, button):
-            return True
-
-        if AXEventSynthesizer.click_object(self.getCurrentAccessible(), button):
-            return True
-
-        return False
-
     def _getCurrentZone(self):
         if not (self.lines and 0 <= self.lineIndex < len(self.lines)):
             return None
@@ -948,6 +854,20 @@ class Context:
 
         return line.zones[self.zoneIndex]
 
+    def getCurrentTextOffset(self):
+        """Returns the current text offset in the current accessible."""
+
+        zone = self._getCurrentZone()
+        if zone is None:
+            return -1
+        if not zone.words:
+            return -1
+        word = zone.words[self.wordIndex]
+        if not word.chars:
+            return -1
+        char = word.chars[self.charIndex]
+        return char.startOffset
+
     def getCurrentAccessible(self):
         """Returns the current accessible."""
 
@@ -1076,7 +996,7 @@ class Context:
 
         return moved
 
-    def goPrevious(self, flatReviewType=ZONE, 
+    def goPrevious(self, flatReviewType=ZONE,
                    wrap=WRAP_ALL, omitWhitespace=True):
         """Moves this context's locus of interest to the first char
         of the previous type.
